#!/bin/bash

# call function with 'tst $arg $expected'
function tst {
  IFS=''
  call="./build/qmc -s \"$1\""
  res=$(./build/qmc -s "$1" | sed -e "s/result: //")
  ./build/qmc -g -s "$1" > table.res
  ./build/qmc -g -s "$2" > table.exp
  res2=$(cmp table.res table.exp)
  if [ $res2 ]; then res3="but table was same" 
  else res3="and table differs"; fi
  rm -f table.res table.exp
  #echo "debug call: $call"
  #echo "debug1 res='$res'"
  if [ "$res" = "$2" ]; then
	  echo ok $1; 
  else
	  echo "false $1 (expected $2) (got $res) ($res3)";
  fi
  #echo
}
# simple logical or
tst 'a+a' 'a'
tst '/a+a' '1'
tst 'a+/a' '1'
tst 'a+b' 'a+b'
tst 'a+b+a' 'a+b'
tst 'a+b+/a' '1'
tst '/a+b+a' '1'
tst 'a+/b+a' 'a+/b'
tst 'a+/b+/a' '1'
tst '/a+/b+a' '1'
tst '/a+/b+/a' '/a+/b'
# simple logical and
tst '/a * a' '0'
tst 'a * /a' '0'
tst 'a * a' 'a'
tst 'aa' 'a'
# simple de morgan
tst '/[a*b]' '/a+/b'
tst '/[a+b]' '/a*/b'
# more complex A
tst '/[a+b]*[/b*c]' '/a*/b*c'
# more complex A inverted by de morgan
tst '[/a*/b] * [/b*c]' '/a*/b*c'
# more complex A inverted twice by de morgan
tst '/[/[/a*/b] + /[/b*c]]' '/a*/b*c'
# more complex A commutative 
tst '[/b*c]*/[a+b]' '/b*c*/a'

# really complex example ;)
# FIXME: tst '[/d*/c*/b*a] + [/d*c*/b*a] + [/d*c*b*a] + [d*/c*/b*/a] + [d*/c*/b*a] + [d*/c*b*a] + [d*c*b*a]' '[b*a] + [d*/c*/b] + [/c*a]'

# example from qmc readme
tst '/[a+b]*[/b*c]+/[a+d]*[/d*e]*[1+/f]*g+[a*b*i*c*d]+[d+/a]' '/a+d'

# example from Wikipedia https://en.wikipedia.org/wiki/Quine%E2%80%93McCluskey_algorithm
tst '[/a*b*/c*/d] + [a*/b*/c*/d] + [a*/b*c*/d]+[a*/b*c*d]+[a*b*/c*/d]+[a*b*c*d]' '[b*/c*/d] + [a*/d] + [a*c]'

